/*jslint nomen: true, indent: 2, maxerr: 3, maxlen: 80*/
/*global window, RSVP, rJS, document*/
(function (window, document, rJS, RSVP) {
  var gadget;
  rJS(window)
  .setState({
    model_gadget : null,
    result_gadget : null
  })

  .declareAcquiredMethod("updateHeader", "updateHeader")
  .declareAcquiredMethod("getUrlParameter", "getUrlParameter")
  .declareAcquiredMethod("getUrlFor", "getUrlFor")
  .declareAcquiredMethod("redirect", "redirect")
  .declareAcquiredMethod("getSetting", "getSetting")
  .declareAcquiredMethod("updatePanel", "updatePanel")
  .declareAcquiredMethod("notifySubmitted", "notifySubmitted")
  .declareAcquiredMethod("getUrlFor", "getUrlFor")

  .ready(function () {
    var model_gadget,
        result_gadget,
        gadget = this,
        about_url;

    return gadget.getDeclaredGadget("model")
    .push(function (model) {
      model_gadget = model;
      return gadget.getDeclaredGadget("result");
    })
    .push(function (result) {
      result_gadget = result;
      return gadget.getUrlFor({command: 'display', options: {
        page: "mynij_about"
      }});
    })
    .push(function (result) {
      about_url = result;
      return model_gadget.getConfig();
    })
    .push(function (config) {
      var proxies = ["https://proxy1.mynij.com/proxy"],
          searx = "https://searx.mynij.com/search",
          store = "https://store.mynij.com";
      if (!config)
        config = {proxies: [], searx: {}, store: {}};
      return gadget.changeState({
        model_gadget : model_gadget,
        result_gadget : result_gadget,
        about_url: about_url,
        proxies: config.proxies.urls || proxies,
        searx: config.searx.url || searx,
        store: config.store.url || store
      });
    });
  })

  .onStateChange(function () {
    var gadget = this,
        msg,
        about = gadget.element.querySelector('.about');
    about.href = gadget.state.about_url;
    if (this.state.notify) {
      msg = gadget.state.notify;
      gadget.state.notify = undefined;
      return gadget.notifySubmitted({
        message: msg,
        status: "success"
      });
    }
  })

  .declareMethod("render", function (options) {
    var gadget = this,
        index_list,
        drop_list = gadget.element.querySelector("#select_index");

    return gadget.changeState({
      notify: options.msg
    })
      .push(function () {
        return RSVP.all([
          gadget.getUrlFor({command: 'history_previous'}),
          gadget.updatePanel({view_action_dict: {}})
        ]);
      })
      .push(function (url_list) {
        return gadget.updateHeader({
          page_title: "Mynij Search"
          //selection_url: url_list[0]
        });
      })
      .push(function () {
        return RSVP.all([gadget.getSetting('index_list', []),
                        gadget.getSetting('build_dict', {})]);
      })
      .push(function (result_list) {
        var add_item = document.createElement("option"),
            elt,
            elt_dict,
            option,
            has_item,
            empty_item;
        index_list = result_list[0];

        while (drop_list.firstChild) {
          drop_list.removeChild(drop_list.firstChild);
        }
        index_list.forEach((item, i) => {
            var option_item = document.createElement("option");
            option_item.value = item;
            option_item.appendChild(document.createTextNode(item));
            drop_list.appendChild(option_item);
        });
        has_item = (index_list.length > 0);
        for (elt in result_list[1]) {
          elt_dict = result_list[1][elt];
          if (!elt_dict.status || elt_dict.status === "not built") {
            option = document.createElement("option");
            option.value = elt_dict.index_name;
            option.appendChild(
              document.createTextNode(elt_dict.index_name + " [building]"));
            option.disabled = true;
            drop_list.appendChild(option);
          }
        }
        if (!has_item) {
          empty_item = document.createElement("option");
          empty_item.style.display = "none";
          drop_list.appendChild(empty_item);
        }
        add_item.value = "ADD";
        add_item.classList.add('add-source');
        add_item.appendChild(document.createTextNode("Add Source..."));
        drop_list.appendChild(add_item);

      })
      .push(function () {
        var limit = options.limit | 100,
            index = options.index;
        if (options.q) {
          if (index_list.indexOf(index) === -1)
            index = undefined;
          else
            drop_list.value = index;
          gadget.element.querySelector("#search_input").value = options.q;
          return gadget.defer_search(options.q, index, limit);
        } else {
          return gadget.clear();
        }
      });
  })

  .declareJob("clear", function () {
    var gadget = this;
    this.element.querySelector("#search_input").value = "";
    return this.state.result_gadget.clear()
      .push(function () {
        var topmenu,
            more;
        document.getElementById("logo-search").classList.remove('hide');
        topmenu = gadget.element.querySelector(".search-mbar");
        more = gadget.element.querySelector(".searx-more");
        if (topmenu)
          topmenu.classList.remove('hide');
        if (more)
          more.classList.add('hide');
        return gadget.state.result_gadget.hideResultBox();
      });
  })

  .declareJob("defer_search", function (search_text, index, limit) {
    var gadget = this,
        top = document.getElementById("logo-search"),
        topmenu = gadget.element.querySelector(".search-mbar"),
        more = gadget.element.querySelector(".searx-more");;

    if(top)
      top.classList.add("hide");
    if (topmenu)
      topmenu.classList.add('hide');
    if (more)
      more.classList.add('hide');
    return gadget.state.result_gadget.startLoading()
      .push(function () {
        return gadget.state.result_gadget.showResultBox();
      })
      .push(function () {
        return RSVP.all[gadget.search(search_text, limit, index),
                        gadget.add_searx_results(search_text)];
      });
  })
  .declareMethod("search", function (key, limit, index) {
    var gadget = this,
        chosen_index;
    if (index) {
      chosen_index = index;
    } else {
      chosen_index = document.getElementById("select_index").value;
    }
    if (chosen_index === null) {
      return gadget.state.result_gadget.addItem(
        {
          title: "Error : No index built",
          link: "",
          body: ""
        },
        key);
    }
    return gadget.state.result_gadget.clear()
      .push(function () {
        console.log("starting search");
        return gadget.state.model_gadget.search(key, chosen_index, limit);
      })
      .push(function (result) {
        var result_queue = new RSVP.Queue(),
            i;
        console.log("search done");
        function push_result(result) {
          result_queue
            .push(function () {
              return gadget.state.model_gadget.get_page(result);
            })
            .push(function (page) {
              return gadget.state.result_gadget.addItem(page, key);
            })
            .push(undefined, function (error) {
              console.log("Error while getting result: " + error);
            });
        }
        if (!result || result.length === 0) {
          return gadget.state.result_gadget.addItem(
            {
              title : "No results found",
              link : ""
            },
            "");
        } else {
          for (i = 0; i < result.length; i += 1) {
            push_result(result[i]);
          }
          return result_queue;
        }
      });
  })

  .declareMethod("add_searx_results", function (key) {
    var gadget = this,
        query = encodeURI(key);

    return new RSVP.Queue()
      .push(function () {
        var count = gadget.state.proxies.length,
            rng = Math.floor(Math.random() * Math.floor(count)),
            proxy = gadget.state.proxies[rng];
        return jIO.util.ajax({
          url : proxy + "?url=" + gadget.state.searx + "%3Fq%3D" +
            query + "%26category_general%3Don%26theme%3Doscar"
        });
      })
      .push(undefined, function () {
        return undefined;
      })
      .push(function (resultHTML) {
        var i,
          promise_list = [],
          elements,
          doc;

        if (!resultHTML) {
          return gadget.state.result_gadget.add_searx_no_result(
            "An error occured while getting result, please try again!"
          );
        }
        doc = new DOMParser().parseFromString(
          resultHTML.currentTarget.response,
          "text/html");
        elements = doc.body.querySelectorAll('div.result.result-default');
        if (elements.length === 0)
          return gadget.state.result_gadget.add_searx_no_result(
            "No result from your input, please update and try again."
          );
        for (i = 0; i < elements.length; i += 1) {
          promise_list.push(gadget.state.result_gadget.show_searx_result(
            elements[i],
            key));
        }
        return new RSVP.Queue()
        .push(function () {
          return RSVP.all(promise_list);
        })
        .push(function () {
          var slink = gadget.state.searx +"?q="+ query + "&category_general=on",
              more = gadget.element.querySelector(".searx-more");
          if (more)
            more.classList.remove('hide');
          return gadget.state.result_gadget.setSearxMoreLink(slink);
        });
      });
  })

  .declareService(function () {
    var gadget = this,
        select = this.element.querySelector(".index-select");
    select.onchange = function (e) {
        e.preventDefault();
        if (e.target.value === "ADD") {
          window.location.href = gadget.state.store;
        }
    };
  })

  .onEvent("submit", function (event) {
    var gadget = this,
        search_key = document.getElementById("search_input").value,
        index = document.getElementById("select_index").value;
    return gadget.redirect({"command": "display", "options": {
      page: "mynij_search",
      q: search_key,
      index: index
    }});
  });
}(window, document, rJS, RSVP));